package Scorch.Weapons;

/*
  Class:  Weapon
  Author: Mikhail Kruk
  Description: the class which is inherited by all the scorch weapons
  used to store explosions, prices, names etc
*/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.*;

import Scorch.*;
import Scorch.Items.*;

public abstract class Weapon extends Item
{
    private static String pkgName = "Scorch.Weapons.";

    public static final int Missile = 0, 
	BabyNuke = 1, Nuke = 2,
	SandBomb = 3, BabyRoller = 4, Roller = 5, HeavyRoller = 6,
	BabyDigger = 7, Digger = 8, HeavyDigger = 9, FunkyBomb = 10,
	Napalm = 11, HotNapalm = 12, MIRV = 13, DeathHead = 14;

    private static final String names[] = {
	"Missile", "Baby Nuke", "Nuke",
	"Sand Bomb", 
	"Baby Roller", "Roller", "Heavy Roller", 
	"Baby Digger", "Digger", "Heavy Digger", 
	"Funky Bomb",
	"Napalm", "Hot Napalm",
	"MIRV", "Death Head"
    };

    protected String explosionClass = "weapon";
    protected int argument = -1;


    // get name of this instance of weapon
    public String getName()
    {
	return names[type];
    }

    // get weapon name by type
    public static String getName(int type)
    {
	return names[type];
    }

    // get type of the weapons buy its name
    public static int getType(String name)
    {
	for(int i = 0; i < names.length; i++)
	    if( names[i].equals(name) )
		return i;
	return -1;
    }

    public ItemControl getControlPanel(ScorchApplet owner)
    {
	controlPanel = new WeaponControl(type, owner);
	return controlPanel;
    }

    public Explosion produceExplosion(Bitmap bitmap, Random rand)
    {
	try
	    {
		// for Netscape again
		Explosion expl = (Explosion)Class.forName
		    (pkgName+explosionClass).newInstance();

		expl.setBitmap(bitmap);
		//expl.setPosition(0,0);
		expl.setRand(rand);
		expl.setArgument(argument);
		return expl;
	    }
	catch(Exception e)
	    {
		System.out.println("produceExplosion: "+e);
	    }
	
	return null;
    }

    // produce a vector of all avaiable weapons. relies on the list of 
    // class names names[]. For now name of the weapons == name of the class
    // (spaces in names are omitted)
    public static Vector loadWeapons()
    {
	Vector result = new Vector();

	try
	    {
		for(int i = 0; i < names.length; i++)
		    {
			String name = trimSpaces(pkgName, names[i]);
			
			result.addElement(Class.forName(name).newInstance());
		    }
	    }
	catch(Exception e)
	    {
		System.out.println("Failed to load weapons: "+e);
	    }
	
	return result;
    }
}

// this class is a panel displayed in the inventory window to control the
// weapon. For most of them (all?) it should be just "select" button
class WeaponControl extends ItemControl implements ActionListener
{
    protected int type;

    public WeaponControl(int type, ScorchApplet owner)
    {
	super(null, owner);
	this.type = type;
	
        control = new Button("Select");
	((Button)control).addActionListener(this);
	
	setLayout(new FlowLayout(FlowLayout.CENTER));
	add(control);
    }

    public void actionPerformed(ActionEvent evt)
    {
	if( evt.getActionCommand().equals("Select") )
	    {
		owner.selectWeapon(type);
	    }
    }
}
